# -*- coding: UTF-8 -*-
import os, sys, traceback, json
from pyopus.misc.debug import DbgSetup, DbgMsgOut
DbgSetup(True, 1)

useMPI=True
mpiData={'cosdebug': 0,
 'mirror': True,
 'persistent': True,
 'processors': None,
 'vmdebug': 0}


projData={'analyses': {'blank': {'command': None,
                        'head': 'opus',
                        'modules': [],
                        'options': {},
                        'params': {},
                        'saves': []},
              'dc': {'command': "dc(-100e-6, 100e-6, 'lin', 100, 'i1', 'dc')",
                     'head': 'opus',
                     'modules': ['def'],
                     'options': {},
                     'params': {},
                     'saves': []}},
 'analysisNames': ['dc', 'blank'],
 'designPar': {'r1': {'hi': 50000.0, 'init': 45000.0, 'lo': 5000.0},
               'r2': {'hi': 200000.0, 'init': 195000.0, 'lo': 20000.0}},
 'designParNames': ['r1', 'r2'],
 'headNames': ['opus'],
 'heads': {'opus': {'moddefs': {'def': {'file': 'bjtamp.inc'}},
                    'options': {},
                    'params': {},
                    'settings': {},
                    'simulator': 'SpiceOpus'}},
 'measureLower': {'gain': 20000.0},
 'measureNames': ['gain', 'nonlinearity'],
 'measureNorm': {'gain': 1000.0, 'nonlinearity': 0.01},
 'measureUpper': {'nonlinearity': 0.08},
 'measures': {'gain': {'analysis': 'dc',
                       'depends': [],
                       'expression': '# Get response\n'
                                     "x, y = scale(), v('out')\n"
                                     '\n'
                                     '# Construct line from first to last '
                                     'point, extract gain\n'
                                     'gain=(y[-1]-y[0])/(x[-1]-x[0])',
                       'vector': False},
              'nonlinearity': {'analysis': 'dc',
                               'depends': [],
                               'expression': '# Get response\n'
                                             "x, y = scale(), v('out')\n"
                                             '\n'
                                             '# Construct line from first to '
                                             'last point\n'
                                             'ylin=y[0]+(x-x[0])*(y[-1]-y[0])/(x[-1]-x[0])\n'
                                             '\n'
                                             '# Maximal deviation from line\n'
                                             'nonlinearity=(np.abs(y-ylin)).max()',
                               'vector': False}},
 'mirrormap': {'2n3510.inc': '.', 'bjtamp.inc': '.'},
 'opPar': {'temperature': {'hi': 100, 'lo': 0, 'nominal': 25},
           'vcc': {'hi': 14, 'lo': 10, 'nominal': 12}},
 'opParNames': ['vcc', 'temperature'],
 'statPar': {},
 'statParNames': [],
 'variableNames': [],
 'variables': {}}

taskData={'cornerNames': ['nominal'],
 'corners': {('nominal', ('opus',)): {'modules': [],
                                      'params': {'temperature': 25,
                                                 'vcc': 12}}},
 'fixedParameterNames': [],
 'name': 'nominal',
 'optParameterNames': ['r1', 'r2'],
 'output': {'aggregatordebug': 0,
            'evaluatordebug': 0,
            'keepfiles': False,
            'optimizerdebug': 0,
            'saveallresults': False,
            'savewaveforms': 'verification',
            'simulatordebug': 0,
            'taskdebug': 1},
 'parameterNames': ['r1', 'r2'],
 'parameters': {'r1': {'hi': 50000.0, 'init': 45000.0, 'lo': 5000.0},
                'r2': {'hi': 200000.0, 'init': 195000.0, 'lo': 20000.0}},
 'requirementNames': ['gain', 'nonlinearity'],
 'requirements': {'exclude': [],
                  'lower': {'gain': 20000.0},
                  'norm': {'gain': 1000.0, 'nonlinearity': 0.01},
                  'tradeoff': {},
                  'upper': {'nonlinearity': 0.08}},
 'settings': {'aggregatorsettings': {},
              'evaluatorsettings': {},
              'failurepenalty': 1000000.0,
              'forwardsolution': True,
              'incrementalcorners': True,
              'initialstep': 0.25,
              'maxiter': None,
              'method': 'ParallelSADE',
              'optimizersettings': {},
              'stopsatisfied': True,
              'stoptol': 1e-05,
              'tradeoffmultiplier': 0},
 'type': 'cbd'}

if __name__=='__main__':
  import numpy as np
  np.random.seed(0)

  if useMPI:
    from pyopus.parallel.mpi import MPI
    from pyopus.parallel.cooperative import cOS

    cOS.setDebug(mpiData['cosdebug'])
    if mpiData['mirror']:
      vm=MPI(mirrorMap=projData['mirrormap'], persistentStorage=mpiData['persistent'], debug=mpiData['vmdebug'])
    else:
      vm=MPI(debug=mpiData['vmdebug'])
    cOS.setVM(vm)

    hosts=MPI.vmStatus['hosts']
    slots=MPI.vmStatus['slots']
    DbgMsgOut('VM', 'Process list (total %d):' % (len(slots)))
    for host in hosts.keys():
      DbgMsgOut('VM', '  Host %s with %d process(es)' % (host, len(hosts[host]['slots'])))
      for slot in hosts[host]['slots']:
        DbgMsgOut('VM', '    slot %d: pid=0x%x (%d)' % (slot, slots[slot]['pid'], slots[slot]['pid']))

  from pyopus.design.sqlite import SQLiteDatabase
  sqld=SQLiteDatabase(os.path.join('..', taskData['name']+'.sqlite'))

  from pyopus.gui.tasks import taskRunner

  try:
    runner=taskRunner[taskData['type']]
    runner(projData, taskData, sqld)
  except Exception as e:
    DbgMsgOut('TASK', 'Task terminated due to exception.')
    DbgMsgOut('TASK', traceback.format_exc())
    sys.exit(1)

  if useMPI:
    cOS.finalize()
